@@ -12,18 +12,22 @@ class Metadata(OrderedDict):
     pass
 
 class Token(OrderedDict):
+    MAPPING = {
+        "upostag": "upos",
+        "xpostag": "xpos",
+        "upos": "upostag",
+        "xpos": "xpostag",
+    }
+
+    def get(self, key, *args, **kwargs):
+        if key not in self and key in self.MAPPING:
+            key = self.MAPPING[key]
+
+        return super(Token, self).get(key, *args, **kwargs)
+
     def __missing__(self, key):
-        if key == "upostag":
-            return self["upos"]
-
-        if key == "xpostag":
-            return self["xpos"]
-
-        if key == "upos":
-            return self["upostag"]
-
-        if key == "xpos":
-            return self["xpostag"]
+        if key in self.MAPPING:
+            return self[self.MAPPING[key]]
 
         raise KeyError("'" + key + "'")
 
