@@ -1,14 +1,16 @@
 from __future__ import unicode_literals
 
 import re
-from collections import OrderedDict, defaultdict
+from collections import OrderedDict
 
 from conllu.compat import fullmatch, text
+from conllu.exceptions import ParseException
+from conllu.models import Metadata, Token
 
-DEFAULT_FIELDS = ('id', 'form', 'lemma', 'upostag', 'xpostag', 'feats', 'head', 'deprel', 'deps', 'misc')
+DEFAULT_FIELDS = ('id', 'form', 'lemma', 'upos', 'xpos', 'feats', 'head', 'deprel', 'deps', 'misc')
 DEFAULT_FIELD_PARSERS = {
     "id": lambda line, i: parse_id_value(line[i]),
-    "xpostag": lambda line, i: parse_nullable_value(line[i]),
+    "xpos": lambda line, i: parse_nullable_value(line[i]),
     "feats": lambda line, i: parse_dict_value(line[i]),
     "head": lambda line, i: parse_int_value(line[i]),
     "deps": lambda line, i: parse_paired_list_value(line[i]),
@@ -70,7 +72,7 @@ def parse_token_and_metadata(data, fields=None, field_parsers=None, metadata_par
         field_parsers = new_field_parsers
 
     tokens = []
-    metadata = OrderedDict()
+    metadata = Metadata()
 
     for line in data.split('\n'):
         line = line.strip()
@@ -91,12 +93,23 @@ def parse_line(line, fields, field_parsers=None):
     # Be backwards compatible if people called parse_line without field_parsers before
     field_parsers = field_parsers or DEFAULT_FIELD_PARSERS
 
+    # Support xpostag/upostag as aliases for xpos/upos (both ways)
+    if "xpostag" not in field_parsers and "xpos" in field_parsers:
+        field_parsers["xpostag"] = field_parsers["xpos"]
+    if "xpos" not in field_parsers and "xpostag" in field_parsers:
+        field_parsers["xpos"] = field_parsers["xpostag"]
+
+    if "upostag" not in field_parsers and "upos" in field_parsers:
+        field_parsers["upostag"] = field_parsers["upos"]
+    if "upos" not in field_parsers and "upostag" in field_parsers:
+        field_parsers["upos"] = field_parsers["upostag"]
+
     line = re.split(r"\t| {2,}", line)
 
     if len(line) == 1:
         raise ParseException("Invalid line format, line must contain either tabs or two spaces.")
 
-    data = OrderedDict()
+    data = Token()
 
     for i, field in enumerate(fields):
         # Allow parsing CoNNL-U files with fewer columns
@@ -221,74 +234,16 @@ def parse_nullable_value(value):
 
     return value
 
-def head_to_token(sentence):
-    if not sentence:
-        raise ParseException("Can't parse tree, need a tokenlist as input.")
+# DEPRECATED: Mantain old paths until next major version
 
-    if "head" not in sentence[0]:
-        raise ParseException("Can't parse tree, missing 'head' field.")
+def serialize(*args, **kwargs):
+    from conllu.serializer import serialize as new_serialize
+    return new_serialize(*args, **kwargs)
 
-    head_indexed = defaultdict(list)
-    for token in sentence:
-        # Filter out range and decimal ID:s before building tree
-        if "id" in token and not isinstance(token["id"], int):
-            continue
+def serialize_field(*args, **kwargs):
+    from conllu.serializer import serialize_field as new_serialize_field
+    return new_serialize_field(*args, **kwargs)
 
-        # Filter out tokens with negative head, they are sometimes used to
-        # specify tokens which should not be included in tree
-        if token["head"] < 0:
-            continue
-
-        head_indexed[token["head"]].append(token)
-
-    if len(head_indexed[0]) == 0:
-        raise ParseException("Found no head node, can't build tree")
-
-    if len(head_indexed[0]) > 1:
-        raise ParseException("Can't parse tree, found multiple root nodes.")
-
-    return head_indexed
-
-def serialize_field(field):
-    if field is None:
-        return '_'
-
-    if isinstance(field, OrderedDict):
-        fields = []
-        for key, value in field.items():
-            if value is None:
-                value = "_"
-
-            fields.append('='.join((key, value)))
-
-        return '|'.join(fields)
-
-    if isinstance(field, tuple):
-        return "".join([serialize_field(item) for item in field])
-
-    if isinstance(field, list):
-        if len(field[0]) != 2:
-            raise ParseException("Can't serialize '{}', invalid format".format(field))
-        return "|".join([serialize_field(value) + ":" + text(key) for key, value in field])
-
-    return "{}".format(field)
-
-def serialize(tokenlist):
-    lines = []
-
-    if tokenlist.metadata:
-        for key, value in tokenlist.metadata.items():
-            if value:
-                line = "# " + key + " = " + value
-            else:
-                line = "# " + key
-            lines.append(line)
-
-    for token_data in tokenlist:
-        line = '\t'.join(serialize_field(val) for val in token_data.values())
-        lines.append(line)
-
-    return '\n'.join(lines) + "\n\n"
-
-class ParseException(Exception):
-    pass
+def head_to_token(*args, **kwargs):
+    from conllu.models import TokenList
+    return TokenList.head_to_token(*args, **kwargs)
